from dataclasses import dataclass
from typing import Any

@dataclass
class DefaultConfig:
    key: str
    value: Any
    desc: str

class DefaultConfigurations:
    def __init__(self, configs=[]):
        self.configs = configs

    def __getitem__(self, config_key):
        return next((c for c in self.configs if c.key == config_key), None)

    def configs_as_dict(self):
        config_dict = dict()
        for config in self.configs:
            config_dict[config.key] = config.value
        return config_dict

    def configuration_keys(self):
        return [config.key for config in self.configs]

@dataclass
class Command:
    command: str
    description: str
    id: str = 0

class CommandArgument:
    def __init__(self, name, required=True):
        self.name = name
        self.required = required

class CommandDetails:
    def __init__(self, responsible_commands, description, command_arguments=[]):
        self.responsible_commands = responsible_commands
        self.description = description
        self.command_arguments = command_arguments

    def need_to_handle(self, command):
       return command in self.responsible_commands 

    def handle(self, args=[]):
        error_message = self.validate(args)
        if error_message:
            raise Exception(error_message)
        else:
            self.handler(args)

    def validate(self, args):
        if len(args) < len(self.get_required_command_arguments()):
            return 'Please provide the correct amount arguments.'

        return ""

    def get_required_command_arguments(self):
        return [a for a in self.command_arguments if a.required]




import os
from cmdcheatsheet.models import DefaultConfigurations, DefaultConfig


user_home_dir = os.path.expanduser('~')

config_dir = f"{user_home_dir}/.config/cmdcheatsheet"
config_name = "config.json"
config_location = f"{config_dir}/{config_name}"

store_name = "commands.json"
default_commands_store_location = f"{config_dir}/{store_name}"

default_config = DefaultConfigurations([
    DefaultConfig(
        "commandsStoreLocation",
        default_commands_store_location,
        'Path to file in JSON format that consists of the command list.'
    ),
    DefaultConfig(
        "alternativeStoreLocations",
        [],
        'Path list to JSON files that might be used as an alternative commands store location.'
    )
])



from setuptools import setup, find_packages
import codecs
import os

curr_location = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(curr_location, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

setup(
    name="cmdcheatsheet",
    version='0.0.14',
    author="obaranovskyi (Oleh Baranovskyi)",
    author_email="<oleh.baranovskyi.dev.acc@gmail.com>",
    description='Terminal commands cheat sheet helper',
    long_description_content_type="text/markdown",
    long_description=long_description,
    url="https://github.com/obaranovskyi/cmdcheatsheet",
    packages=find_packages(),
    install_requires=['rich'],
    keywords=['python', 'command', 'terminal', 'console', 'utilities'],
    entry_points={
        'console_scripts': [
            'cmdcheatsheet=cmdcheatsheet.cmdcheatsheet:main'
        ]
    },
    classifiers=[
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS",
        "Topic :: Utilities"
    ]
)



import json
from rich.prompt import Confirm
from cmdcheatsheet.consts import default_config
from cmdcheatsheet.messages import show_invalid_store_location_message

def validate_configuration(config_key, config_value):
    exists = config_key in default_config.configuration_keys()
    valid = True
    if not exists:
        valid = Confirm.ask(
            f"The configuration with the name '{config_key}' doesn't exist." +
            " Therefore, it won't have any impact. Do you still want to add it?"
        )

    if config_key == 'commandsStoreLocation' and not is_valid_custom_commands_location(config_value):
        show_invalid_store_location_message()
        return False
    return valid

def is_valid_custom_commands_location(store_location):
    try:
        with open(store_location) as f:
            commands = json.load(f)
            return isinstance(commands, list)
    except Exception as _: 
        return False
